/* Principal.m
 * load gmodel files listed in plist and convert them as specified
 *
 * Copyright (C) 2001 by vhf interservice GmbH
 * Author:  Georg Fleischmann
 *
 * created:  2001-07-22
 * modified: 
 *
 * Usage: convertgmodels plistFile
 *
 * This tool is free software; you can redistribute it and/or
 * modify it under the terms of the Lesser GNU Public License (LGPL).
 * Among other things, the License requires that the copyright
 * notices and this notice be preserved on all copies.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Public License for more details.
 *
 * http://www.vhf.de
 */

#import "Principal.h"

@interface Principal(PrivateMethods)
@end

@implementation Principal

/*
 * created:  2001-07-22
 * modified: 
 */
- (void)convert
{   NSArray		*args = [[NSProcessInfo processInfo] arguments], *keys, *languages;
    NSDictionary	*dict;
    NSString		*path;
    int			i, o, k, l;

    if ([args count]<=1)
    {   printf("usage: convertgmodels plistFile");
        return;
    }
    dict = [NSDictionary dictionaryWithContentsOfFile:[args objectAtIndex:1]];
    if (!dict)
    {   NSLog(@"Couldn't load property list: %@", [args objectAtIndex:1]);
        return;
    }

    NSLog(@"*** convertgmodels %@ ***", [args objectAtIndex:1]);

    keys = [dict allKeys];
    path = [[args objectAtIndex:1] stringByDeletingLastPathComponent];
    languages = [dict objectForKey:@"languages"];

    /* files */
    for (i=0; i<[keys count]; i++)
    {   NSString	*fileMask = [path stringByAppendingPathComponent:[keys objectAtIndex:i]];

        if (![fileMask hasSuffix:@".gmodel"])
            continue;

        /* languages */
        for (l=0; l<[languages count]; l++)
        {   NSAutoreleasePool	*pool = [NSAutoreleasePool new];
            NSString		*fileName = [NSString stringWithFormat:fileMask, [languages objectAtIndex:l]];
            NSDictionary	*objectsDict = [dict objectForKey:[keys objectAtIndex:i]];
            NSArray		*objectsKeys = [objectsDict allKeys], *keys;
            NSMutableDictionary	*gmodel;

            gmodel = [NSMutableDictionary dictionaryWithContentsOfFile:fileName];
            if (!gmodel)
                NSLog(@"gmodel file does not exist: %@", fileName);

            /* correct general stuff */
            keys = [gmodel allKeys];
            for (o=0; o<[keys count]; o++)
            {   NSMutableDictionary	*obj = [gmodel objectForKey:[keys objectAtIndex:o]];

                if (!obj || ![obj isKindOfClass:[NSDictionary class]])
                    continue;
                /* isa = NSImage, name = NSSwitch, NSRadioButton, ... -> remove size */
                if ( [[obj objectForKey:@"isa"] isEqual:@"NSImage"] &&
                     ( [[obj objectForKey:@"name"] isEqual:@"NSSwitch"]            ||
                       [[obj objectForKey:@"name"] isEqual:@"NSHighlightedSwitch"] ||
                       [[obj objectForKey:@"name"] isEqual:@"NSRadioButton"]       ||
                       [[obj objectForKey:@"name"] isEqual:@"NSHighlightedRadioButton"] ) )
                {   obj = [[obj mutableCopy] autorelease];
                    [obj removeObjectForKey:@"size"];
                    [gmodel setObject:obj forKey:[keys objectAtIndex:o]];
                }
            }

            /* replace entries from dictionary */
            for (o=0; o<[objectsKeys count]; o++)
            {   NSMutableDictionary	*obj = [[gmodel objectForKey:[objectsKeys objectAtIndex:o]] mutableCopy];
                NSDictionary		*replaceDict = [objectsDict objectForKey:[objectsKeys objectAtIndex:o]];
                NSArray			*replaceKeys = [replaceDict allKeys];

                if (!obj)
                    continue;
                [gmodel setObject:[obj autorelease] forKey:[objectsKeys objectAtIndex:o]];
                for (k=0; k<[replaceKeys count]; k++)
                {   NSString	*key = [replaceKeys objectAtIndex:k];
                    [obj setObject:[replaceDict objectForKey:key] forKey:key];
                    NSLog(@"%@: %@ -> %@ = %@", fileName, [objectsKeys objectAtIndex:o], key, [replaceDict objectForKey:key]);
                }
            }
            [gmodel writeToFile:fileName atomically:YES];
            [pool release];
        }
    }
}

@end
